///////////////////////////////////////////////////////////////
// File Name: hc11.h
// Purpose:  Definition of the main Hc11 class
///////////////////////////////////////////////////////////////

#ifndef HC11_H
#define HC11_H

#include "basetypes.h"
#include "register.h"
#include "regfile.h"
#include "hc11io.h"
#include "memory.h"
#include "timer.h"
#include "iosystem.h"
#include "hc11misc.h"

#define N_OPCODES 256


// some interrupt stuff

#define N_IBIT_INTERRUPTS 16

/*
#define SWIVector   (0xF4)
#define XIRQVector  (0xF4)
#define IRQVector   (0xF2)
#define RTIVector   (0xF0)
#define IC1Vector   (0xEE)
#define IC2Vector   (0xEC)
#define IC3Vector   (0xEA)
#define OC1Vector   (0xE8)
#define OC2Vector   (0xE6)
#define OC3Vector   (0xE4)
#define OC4Vector   (0xE2)
#define I4O5Vector  (0xE0)
#define TOVector    (0xDE)
#define PAOVVector  (0xDC)
#define PAIVector   (0xDA)
*/

//changed base vector from ff00 to ffco and b stuff.
#define ResetVector (0x3E)
#define IllegalVector (0x38)
#define SWIVector   (0x36)
#define XIRQVector  (0x34)
#define IRQVector   (0x32)
#define RTIVector   (0x30)
#define IC1Vector   (0x2E)
#define IC2Vector   (0x2C)
#define IC3Vector   (0x2A)
#define OC1Vector   (0x28)
#define OC2Vector   (0x26)
#define OC3Vector   (0x24)
#define OC4Vector   (0x22)
#define I4O5Vector  (0x20)
#define TOVector    (0x1E)
#define PAOVVector  (0x1C)
#define PAIVector   (0x1A)


// maybe make an opcode_t struct with include the cycle
typedef void (*opcode_t)(class HC11 *);
typedef bool (*interrupt_service_t)(class HC11 *);

class HC11
{     
 // MC68HC11 CPU
 public:
	// ACCD, ACC.A and ACC.D     
    // ACCD is (A<<8)|:B
	Byte	ACCA;	
    Byte	ACCB;		
	// The condition code register
	
    CCRReg CCR;	        

	// Index Registers
	Word	IX;
	Word	IY;
	// Program Counter
	Word	PC;
	// Stack Pointer
	Word	SP;
    
    // continuous counter
	long int timer_count;
   	
  	
 // Misc Pins    
    void XIRQLevel(bit_t);
    void IRQLevel(bit_t);    
    bit_t XIRQLevel(void);
    bit_t IRQLevel(void);
 
    Pin STRA;   //handshake io stuff
    Pin STRB;

    // check if IRQ interrupt occured
    bool IRQInterrupt(void);
    
 // The Memory Map
    MemoryMap	memory;

 // Some memory map objects 
	RegisterMap* regmap;
	RAM* chip_ram;
	ROM* boot_rom;

 // The Memory Map
    RegisterFile    regfile;

 // Subsystem Stuff    
    TimerSubSystem	*timer_system;  
    HandshakeIOSubSystem *handshake_system;
    // ADSubSystem	*ad_system;
    // SPISubSystem	*spi_system;
    // SCISubSystem	*sci_system;    
    friend class TimerSubSystem; 
        
 private:        
    bool xbit_cleared;
    // used in the STOP and WAIT opcodes    
    bool stopped;
    bool waiting;

    bit_t xirq_level;
    bit_t irq_level;
    bit_t irq_edge;
    // this is the internal clocking function
    void _clock(void); 	
	void LoadROMBootLoader(word_t vector_base_addr);  //for bootstrap mode     
 protected:

 // Opcodes Stuff                               
    // the array of opcode function pointers    
    static opcode_t opcode[N_OPCODES];
    static opcode_t opIYcode[N_OPCODES];
    // executes the instruction pointed to by PC
    void Fetch();
    // this would set PC to address and grab the opcode
    void Fetch(word_t address);
        
    // opcode declarations
    #include "opcodes.h"
               
 // Interrupt Stuff   
    // priority resolution    
    void ResolveInterrupt(void);	
    // the base address of the interrupt vectors    
    word_t  vector_base_addr;
    void FetchVector(word_t num);
    
    // checked at the end of the current opcode execution        
    inline bool InterruptPending(void)
    {            
        return(IRQInterrupt() || (regfile.TFLG1)&(regfile.TMSK1) || 
                                 (regfile.TFLG2)&(regfile.TMSK2));
        // also check for spi/sci later
    };

    
    // used for priority ordering (required for PSEL)        
    static interrupt_service_t psel_table[N_IBIT_INTERRUPTS];    
    // these service the interrupt if it occured
    static bool ServiceTO(HC11 *hc11);
    static bool ServicePAOV(HC11 *hc11);
    static bool ServicePAI(HC11 *hc11);
    static bool ServiceSPI(HC11 *hc11);
    static bool ServiceSCI(HC11 *hc11);
    static bool ServiceIRQ(HC11 *hc11);    
    static bool ServiceIRQIO(HC11 *hc11);    
    static bool ServiceRTI(HC11 *hc11);
    static bool ServiceIC1(HC11 *hc11);
    static bool ServiceIC2(HC11 *hc11);
    static bool ServiceIC3(HC11 *hc11);
    static bool ServiceOC1(HC11 *hc11);
    static bool ServiceOC2(HC11 *hc11);
    static bool ServiceOC3(HC11 *hc11);
    static bool ServiceOC4(HC11 *hc11);
    static bool ServiceI4O5(HC11 *hc11);
    
 // Interface Functions
 public:    
    HC11();
    ~HC11();

    // changing this will only take effect after a reset
    Mode mode;
        
    void Reset(void);
    void Step(void); 

    friend class PortARegister;
};

#endif

