#include "basetypes.h"
#include "memory.h"
#include "hc11misc.h"
#include "list.h"
#include "regfile.h"

#define SIZE_OF_REGISTER_BLOCK 0x3f


MemoryObject::MemoryObject()
{
    enable = true;
}

void MemoryObject::Move(word_t new_base)
{
    int new_top;
        
    new_top = new_base + size;
    // maximum address is 0xffff, so truncate this block at 0xffff    
    if(new_top > 0xffff)
        new_top = 0xffff;

    base_addr = new_base;
    top_addr = new_top;
}


///////////////////////////////////////////////////
// Register Map Functions
///////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////
//
// Function Name: RegisterMap::RegisterMap()
//
// Author: Paul Clarke
//
// Purpose: 
//          
// Input Parameters:    word_t base,
//                      RegisterFile* regfile
//
// Return Value: None
//
/////////////////////////////////////////////////////////////////////////////
RegisterMap::RegisterMap(word_t base,RegisterFile* regfile):MemoryObject()
{	
    
	base_addr = base;
	top_addr = base_addr + SIZE_OF_REGISTER_BLOCK;
    size = SIZE_OF_REGISTER_BLOCK;

	// Assign the array of pointers to the register objects

	regs[0x00] = &regfile->PORTA;
	regs[0x01] = &reserved;
	regs[0x02] = &regfile->PIOC;
	// Reserved at 0x03
	regs[0x03] = &regfile->PORTC;
	regs[0x04] = &regfile->PORTB;
	// should be port CL (paul didn't implement this guy) bad paul bad paul
	regs[0x05] = &regfile->PORTC;
	// Reserved at 0x06
	regs[0x06] = &reserved;
	regs[0x07] = &regfile->DDRC;
	regs[0x08] = &regfile->PORTD;
	regs[0x09] = &regfile->DDRD;
	regs[0x0a] = &regfile->PORTE;
	regs[0x0b] = &regfile->CFORC;
	regs[0x0c] = &regfile->OC1M;
	regs[0x0d] = &regfile->OC1D;
	regs[0x0e] = &regfile->TCNT.high;
	regs[0x0f] = &regfile->TCNT.low;
	regs[0x10] = &regfile->TIC1.high;
	regs[0x11] = &regfile->TIC1.low;
	regs[0x12] = &regfile->TIC2.high;
	regs[0x13] = &regfile->TIC2.low;
	regs[0x14] = &regfile->TIC3.high;
	regs[0x15] = &regfile->TIC3.low;
	regs[0x16] = &regfile->TOC1.high;
	regs[0x17] = &regfile->TOC1.low;
	regs[0x18] = &regfile->TOC2.high;
	regs[0x19] = &regfile->TOC2.low;
	regs[0x1a] = &regfile->TOC3.high;
	regs[0x1b] = &regfile->TOC3.low;
	regs[0x1c] = &regfile->TOC4.high;
	regs[0x1d] = &regfile->TOC4.low;
	regs[0x1e] = &regfile->TI4O5.high;
	regs[0x1f] = &regfile->TI4O5.low;
	regs[0x20] = &regfile->TCTL1;
	regs[0x21] = &regfile->TCTL2;
	regs[0x22] = &regfile->TMSK1;
	regs[0x23] = &regfile->TFLG1;
	regs[0x24] = &regfile->TMSK2;
	regs[0x25] = &regfile->TFLG2;
	regs[0x26] = &regfile->PACTL;
	regs[0x27] = &regfile->PACNT;
	regs[0x28] = &regfile->SPCR;
	regs[0x29] = &regfile->SPSR;
	regs[0x2a] = &regfile->SPDR;
	regs[0x2b] = &regfile->BAUD;
	regs[0x2c] = &regfile->SCCR1;
	regs[0x2d] = &regfile->SCCR2;
	regs[0x2e] = &regfile->SCSR;
	regs[0x2f] = &regfile->SCDR;
	regs[0x30] = &regfile->ADCTL;
	regs[0x31] = &regfile->ADR1;
	regs[0x32] = &regfile->ADR2;
	regs[0x33] = &regfile->ADR3;
	regs[0x34] = &regfile->ADR4;
	regs[0x35] = &regfile->BPROT;
	regs[0x36] = &regfile->EPROG;
	// Reserved at 0x37
	regs[0x37] = &reserved;
	// Reserved at 0x38
	regs[0x38] = &reserved;
	regs[0x39] = &regfile->OPTION;
	regs[0x3a] = &regfile->COPRST;
	regs[0x3b] = &regfile->PPROG;
	regs[0x3c] = &regfile->HPRIO;
	regs[0x3d] = &regfile->INIT;
	regs[0x3e] = &regfile->TEST1;
	regs[0x3f] = &regfile->CONFIG;

}

RegisterMap::~RegisterMap()
{
}

byte_t RegisterMap::Read(word_t address)
{
	return(regs[address-base_addr]->Read());
}

void RegisterMap::Write(word_t address, byte_t data)
{
	assert(address >= base_addr);
	regs[address-base_addr]->Write(data);
}


///////////////////////////////////////////////////
// RAM Functions
///////////////////////////////////////////////////
RAM::RAM(word_t base,int mem_size):MemoryObject()
{

	base_addr = base;
    size = mem_size;
	top_addr = base_addr + size;

	// Allocate memory
	mem = new byte_t[size];
    assert(mem!=NULL);
}

RAM::~RAM ()
{
	// Free memory
    if(mem!=NULL)
	    delete[] mem;
}
byte_t RAM::Read(word_t address)
{
    return(mem[address-base_addr]);
}	
void RAM::Write(word_t address, byte_t data)
{    
    mem[address-base_addr]=data;
}


ROM::ROM(word_t base,int mem_size):MemoryObject()
{
	assert((mem_size >=0) && (mem_size <= 0xffff));

	base_addr = base;
    size = mem_size;
	top_addr = base_addr + size;

	// Allocate memory
	mem = new byte_t[size];
    assert(mem!=NULL);
}

ROM::~ROM ()
{
	// Free memory
	if(mem!=NULL)
	    delete[] mem;
}

byte_t ROM::Read(word_t address)
{
  return(mem[address-base_addr]);
};

void ROM::Program(word_t address, byte_t data)
{
	mem[address-base_addr] = data;
}

///////////////////////////////////////////////////
// Memory Map Functions
///////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Function Name: MemoryMap::GetObject
//
// Author: Kalle Anderson
//
// Purpose: Finds the first MemoryObject containing the address and returns it.
//          
// Input Parameters: address
//
// Return Value: The memory object, or NULL if it wasn't found.
//
/////////////////////////////////////////////////////////////////////////////

MemoryObject *MemoryMap::GetObject(word_t address)
{
	MemoryObject *mo;
	list_iter<MemoryObject *> index;

    // this will speed stuff up if there are a lot of continuous
    // accesses to the same region of memory, although I just
    // realized it will screw up the memory priority stuff, if
    // an internal and external block of memory are overlapping
    // so that is why it is comment out
//    if((mo_cache!=NULL) && mo_cache->Contains(address) && mo_cache->enable)		
//			return(mo_cache);		        
    
    index = map.first();

	while(index.is_valid())
	{
		mo = map[index];
		if(mo->Contains(address) && mo->enable)		
        {
            mo_cache=mo;
			return(mo);		
        }
		index++;
	}	
	
	log("INVALID MEMORY ACCESS @ %X!\n",address);
	return (NULL);
}


//////////////////////////////////////////////////////////////////////////////
//
// Function Name: MemoryMap::Read()
//
// Author: Kalle Anderson
//
// Purpose: Reads a memory address. If there is no object containing that
//          address, the thing it returns, is intenitally uninitialized garbage.
//          
// Input Parameters: address - the address
//
// Return Value: byte_t - the value read from memory
//
/////////////////////////////////////////////////////////////////////////////
byte_t MemoryMap::Read(word_t address)
{
    byte_t data;
    MemoryObject *mo = GetObject(address);

    if(mo)
	    data = mo->Read(address);

    return(data);
}


//////////////////////////////////////////////////////////////////////////////
//
// Function Name: MemoryMap::Write
//
// Author: Kalle Anderson
//
// Purpose: write a byte to a memory location in the map. This finds the 
//          first memory object to contain the address, and writes the 
//          byte to it. If there is no memory object, it does nothing.
//                    
// Input Parameters: word_t address
//                   byte_t data
//
// Return Value: None
//
/////////////////////////////////////////////////////////////////////////////
void MemoryMap::Write(word_t address, byte_t data)
{    
    MemoryObject *mo = GetObject(address);

    if(mo)
	    mo->Write(address,data);
};

//////////////////////////////////////////////////////////////////////////////
// Function Name: MemoryMap::AddMemoryObject
//
// Author: Kalle Anderson
//
// Purpose: Add an object to the MemoryMap
//          
// Input Parameters: MemoryObject *mo - pointer to the object
//
// Return Value: None
/////////////////////////////////////////////////////////////////////////////
void MemoryMap::AddMemoryObject(MemoryObject *mo)
{
	map.add(mo);
};


//////////////////////////////////////////////////////////////////////////////
// Function Name: MemoryMap::RemoveMemoryObject
//
// Author: Kalle Anderson
//
// Purpose: Remove an object from the MemoryMap
//          
// Input Parameters: MemoryObject *mo - pointer to the object to remove
//
// Return Value: bool - if it found the object
/////////////////////////////////////////////////////////////////////////////
bool MemoryMap::RemoveMemoryObject(MemoryObject *mo)
{	
	list_iter<MemoryObject *> index;

    index = map.first();
	while(index.is_valid())
	{
		if(mo == map[index])
        {
            if(mo == mo_cache)
                mo_cache = NULL;
            map.remove(index);            
            return(true);        
		}
		index++;
	}		
	return (false);
};


//////////////////////////////////////////////////////////////////////////////
// Function Name: MemoryMap::MemoryMap()
// Purpose: None
/////////////////////////////////////////////////////////////////////////////
MemoryMap::MemoryMap()
{
    // nothing needs to be done here at this point
    mo_cache=NULL;
}

//////////////////////////////////////////////////////////////////////////////
// Function Name: MemoryMap::~MemoryMap()
//
// Author: Kalle Anderson
//
// Purpose: Destructor for the MemoryMap. This actually deletes all the memory
//          maps that it contains. 
//          
// Input Parameters: None
//
// Return Value: None
/////////////////////////////////////////////////////////////////////////////

MemoryMap::~MemoryMap()
{
	// Clean out the list
	MemoryObject *mo;
	list_iter<MemoryObject *> index;
	
    index = map.first();
	while(index.is_valid())
	{
		mo = map[index];
		delete mo;
		index++;
	}	
}



