#ifndef MEMORY_H
#define MEMORY_H

///////////////////////////////////////////////////////////////
// File Name: memory.h
// Purpose:  Definition of memory objects
///////////////////////////////////////////////////////////////

#include "basetypes.h"
#include "register.h"
#include "list.h"
#include <assert.h>


#define NUM_REGISTERS 64


//////////////////////////////////////////////////////////////////////////////
//
// Class Name: MemoryObject
//
// Author: Kalle Anderson
//
// Purpose: The base class for all the MemoryObjects. A memory object represents
//          a type of memory, like RAM or ROM. Each memory object is assigned
//          a certain range of memory.
//          
/////////////////////////////////////////////////////////////////////////////
class MemoryObject
{
 protected:
    word_t size;
	word_t base_addr;
	word_t top_addr;				
 public:
    bit_t enable;

    MemoryObject();
    virtual ~MemoryObject(){;};
	virtual byte_t Read(word_t address)=0;
	virtual void Write(word_t address, byte_t data)=0;	
    bool Contains(word_t addr){return((addr>=base_addr)&&(addr<=top_addr));};
    void Move(word_t new_base);    
};

//////////////////////////////////////////////////////////////////////////////
//
// Class Name: RegisterMap
//
// Author: Kalle Anderson
//
// Purpose: The range of memory that maps all the HC11 ports/registers.
//          
/////////////////////////////////////////////////////////////////////////////
class RegisterMap: public MemoryObject
{
 private:
	MemoryRegister *regs[NUM_REGISTERS];
	ByteRegister reserved;
 public:

	RegisterMap(word_t base, class RegisterFile* regfile);
	~RegisterMap();
	byte_t Read(word_t address);	
	void Write(word_t address, byte_t data);	
};


//////////////////////////////////////////////////////////////////////////////
//
// Class Name: RegisterMap
//
// Author: Kalle Anderson
//
// Purpose: The generic RAM memory class.
//          
/////////////////////////////////////////////////////////////////////////////
class RAM: public MemoryObject
{
 private: 	    
	byte_t *mem;    
 public:	        
    RAM(word_t base,int mem_size);
    ~RAM();
	byte_t Read(word_t address);	
    void Write(word_t address, byte_t data);
};

//////////////////////////////////////////////////////////////////////////////
//
// Class Name: RegisterMap
//
// Author: Kalle Anderson
//
// Purpose: The generic ROM class. Can only be written to using the Program 
//          function.
//          
/////////////////////////////////////////////////////////////////////////////
class ROM: public MemoryObject
{
private:
	byte_t *mem;    
public:	
    ROM(word_t base,int mem_size);
    ~ROM();
	byte_t Read(word_t address);
	void Write(word_t address, byte_t data){};
    void Program(word_t address, byte_t data);
};

//////////////////////////////////////////////////////////////////////////////
//
// Class Name: MemoryMap
//
// Author: Kalle Anderson
//
// Purpose: This class maps all the different MemoryObjects into a
//          contiguous address space. 
//          
/////////////////////////////////////////////////////////////////////////////
class MemoryMap
{	
private:
    list<MemoryObject*> map;
	MemoryObject *GetObject(word_t address);

    MemoryObject *mo_cache;

public:	
	 ~MemoryMap();
	 MemoryMap();

	byte_t Read(word_t address);
	void Write(word_t address, byte_t data);
    // the order of objects added will determine there priority
    // if they overlap
	void AddMemoryObject(MemoryObject *mo);    
    bool RemoveMemoryObject(MemoryObject *mo);
    byte_t operator[](word_t address){return(Read(address));};
};

#endif